////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// BeDVI                                                                                                          //
// by Achim Blumensath                                                                                            //
// blume@corona.oche.de                                                                                           //
//                                                                                                                //
// This program is free software! It may be distributed according to the GNU Public License (see COPYING).        //
//                                                                                                                //
// $Id: BeDVI.cc,v 1.4 1997/04/27 17:41:29 achim Exp achim $
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <InterfaceKit.h>
#include <StorageKit.h>
#include <NodeInfo.h>
#include <Path.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "defines.h"

extern "C"
{
  #define string _string
  #include "kpathsea/progname.h"
  #include "kpathsea/proginit.h"
  #include "kpathsea/tex-file.h"
  #undef string
};

#include "debug.h"
#include "BeDVI.h"
#include "DVI-Window.h"
#include "DVI-View.h"
#include "FontList.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// ViewApplication::ViewApplication()                                                                             //
//                                                                                                                //
// initializes the display and global data (see the BeBook).                                                      //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

ViewApplication::ViewApplication():
  BApplication("application/x-vnd.Be-BDVI"),
  win(NULL),
  vw(NULL),
  WindowBounds(90.0, 30.0, 630.0, 470.0),
  OpenPanel(NULL)
{
  app_info   AppInfo;
  BEntry     AppFileEntry;
  BPath      AppPath;
  BNode      AppFile;
  u_int      PixelsPerInch = 300;
  u_int      ShrinkFactor  = 3;
  char       Buffer[1024];

  D(bug("ViewApplication::ViewApplication()\n"));

  // read preferences

  Settings.AntiAliasing = true;
  Settings.BorderLine   = false;

  if(GetAppInfo(&AppInfo) != B_OK)
    return;

  if(AppFileEntry.SetTo(&AppInfo.ref) != B_NO_ERROR)
    return;

  if(AppFileEntry.GetPath(&AppPath) != B_NO_ERROR)
    return;

  acquire_sem(kpse_sem);
  kpse_set_progname(AppPath.Path());    // set default name
  release_sem(kpse_sem);

  if(AppFile.SetTo(&AppInfo.ref) != B_NO_ERROR)
    return;

  AppFile.Lock();

  if(AppFile.ReadAttr("BeDVI: win x1",       B_FLOAT_TYPE, 0, Buffer, 1020) > 0)
    memcpy(&WindowBounds.left,              Buffer, sizeof(WindowBounds.left));
  if(AppFile.ReadAttr("BeDVI: win y1",       B_FLOAT_TYPE, 0, Buffer, 1020) > 0)
    memcpy(&WindowBounds.top,               Buffer, sizeof(WindowBounds.top));
  if(AppFile.ReadAttr("BeDVI: win x2",       B_FLOAT_TYPE, 0, Buffer, 1020) > 0)
    memcpy(&WindowBounds.right,             Buffer, sizeof(WindowBounds.right));
  if(AppFile.ReadAttr("BeDVI: win y2",       B_FLOAT_TYPE, 0, Buffer, 1020) > 0)
    memcpy(&WindowBounds.bottom,            Buffer, sizeof(WindowBounds.bottom));
  if(AppFile.ReadAttr("BeDVI: antialiasing", B_BOOL_TYPE,  0, Buffer, 1020) > 0)
    memcpy(&Settings.AntiAliasing,          Buffer, sizeof(Settings.AntiAliasing));
  if(AppFile.ReadAttr("BeDVI: borderline",   B_BOOL_TYPE,  0, Buffer, 1020) > 0)
    memcpy(&Settings.BorderLine,            Buffer, sizeof(Settings.BorderLine));
  if(AppFile.ReadAttr("BeDVI: dpi",          B_INT32_TYPE, 0, Buffer, 1020) > 0)
    memcpy(&Settings.DspInfo.PixelsPerInch, Buffer, sizeof(Settings.DspInfo.PixelsPerInch));
  if(AppFile.ReadAttr("BeDVI: shrink",       B_INT32_TYPE, 0, Buffer, 1020) > 0)
    memcpy(&Settings.DspInfo.ShrinkFactor,  Buffer, sizeof(Settings.DspInfo.ShrinkFactor));

  AppFile.Unlock();

  SetResolution(PixelsPerInch, ShrinkFactor);

  getcwd(Buffer, 1024);                             // there apparently is no other way to get the current directory!
  get_ref_for_path(Buffer, &PanelDirectory);

/*  if(!InitAppMenu())
  {
    DisplayError("Not enough memory!");
    return;
  }*/
  if(!(win = InitWindow()))
  {
    DisplayError("Can't create window!");
    return;
  }
  vw = (DVIView *)win->FindView("DVI-View");

  vw->UpdateMenus();

  win->Show();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// ViewApplication::~ViewApplication()                                                                            //
//                                                                                                                //
// saves current settings in executable resources (see the BeBook).                                               //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

ViewApplication::~ViewApplication()
{
  app_info AppInfo;
  BNode    AppFile;

  D(bug("ViewApplication::~ViewApplication()\n"));

  if(OpenPanel)
    delete OpenPanel;

  if(GetAppInfo(&AppInfo) != B_NO_ERROR)
    return;

  if(AppFile.SetTo(&AppInfo.ref) != B_NO_ERROR)
    return;

  AppFile.Lock();
  AppFile.RemoveAttr("BeDVI: win x1");
  AppFile.RemoveAttr("BeDVI: win y1");
  AppFile.RemoveAttr("BeDVI: win x2");
  AppFile.RemoveAttr("BeDVI: win y2");
  AppFile.RemoveAttr("BeDVI: antialiasing");
  AppFile.RemoveAttr("BeDVI: borderline");
  AppFile.RemoveAttr("BeDVI: dpi");
  AppFile.RemoveAttr("BeDVI: shrink");
  AppFile.WriteAttr("BeDVI: win x1",       B_FLOAT_TYPE, 0, &WindowBounds.left,              sizeof(WindowBounds.left));
  AppFile.WriteAttr("BeDVI: win y1",       B_FLOAT_TYPE, 0, &WindowBounds.top,               sizeof(WindowBounds.top));
  AppFile.WriteAttr("BeDVI: win x2",       B_FLOAT_TYPE, 0, &WindowBounds.right,             sizeof(WindowBounds.right));
  AppFile.WriteAttr("BeDVI: win y2",       B_FLOAT_TYPE, 0, &WindowBounds.bottom,            sizeof(WindowBounds.bottom));
  AppFile.WriteAttr("BeDVI: antialiasing", B_BOOL_TYPE,  0, &Settings.AntiAliasing,          sizeof(Settings.AntiAliasing));
  AppFile.WriteAttr("BeDVI: borderline",   B_BOOL_TYPE,  0, &Settings.BorderLine,            sizeof(Settings.BorderLine));
  AppFile.WriteAttr("BeDVI: dpi",          B_INT32_TYPE, 0, &Settings.DspInfo.PixelsPerInch, sizeof(Settings.DspInfo.PixelsPerInch));
  AppFile.WriteAttr("BeDVI: shrink",       B_INT32_TYPE, 0, &Settings.DspInfo.ShrinkFactor,  sizeof(Settings.DspInfo.ShrinkFactor));
  AppFile.Unlock();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::ArgvReceived(int32 argc, char **argv)                                                    //
//                                                                                                                //
// Parses command line arguments (see the BeBook).                                                                //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::ArgvReceived(int32 argc, char **argv)
{
  if(argc > 2)
  {
    printf("USAGE: BeDVI [<file>]\n");
    Quit();
    return;
  }
  if(argc == 2)
  {
    entry_ref file;

/*    if(argv[1][0] != '/')
    {
      char *Buffer;
      
      if(!(Buffer = new char[4096]))
      {
        printf("not enough memory!\n");
        Quit();
        return;
      }
      getcwd(Buffer, 4096);
      strncat(Buffer, "/",     4096);
      strncat(Buffer, argv[1], 4096);

      if(get_ref_for_path(Buffer, &file) != B_NO_ERROR)
      {
        delete [] Buffer;
        printf("can't open file!\n");
        Quit();
        return;
      }
      delete [] Buffer;
    }
    else*/
      if(get_ref_for_path(argv[1], &file) != B_NO_ERROR)
      {
        printf("can't open file!\n");
        Quit();
        return;
      }

    OpenFile(file);
  }
  acquire_sem(kpse_sem);
  kpse_set_progname(argv[0]);
  release_sem(kpse_sem);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::RefsReceived(BMessage *msg)                                                              //
//                                                                                                                //
// Loads a file (see the BeBook).                                                                                 //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::RefsReceived(BMessage *msg)
{
  entry_ref file;

  if(msg->FindRef("refs", 0, &file) < B_NO_ERROR)
    return;

  OpenFile(file);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// bool ViewApplication::QuitRequested()                                                                          //
//                                                                                                                //
// prepares for shutdown (see the BeBook).                                                                        //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool ViewApplication::QuitRequested()
{
  D(bug("ViewApplication::QuitRequested()\n"));

  if(OpenPanel)
  {
    delete OpenPanel;                                      // necessary to prevent deadlock when closing the window
    OpenPanel = NULL;
  }

  BApplication::QuitRequested();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::AboutRequested()                                                                         //
//                                                                                                                //
// Displays the About Requester (see the BeBook).                                                                 //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::AboutRequested()
{
  BAlert *al;

  if(!(al = new BAlert("About",
                       "BeDVI " VERSION " ("__DATE__")\n\n"
                       "by Achim Blumensath\n\n"
                       "This program is free software!\n"
                       "Consult the GNU Public License for more information.",
                       "Ok", NULL, NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT)))
    return;

  al->Go();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::MessageReceived(BMessage *msg)                                                           //
//                                                                                                                //
// Loads the selected file if the FilePanel was closed (see the BeBook).                                          //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::MessageReceived(BMessage *msg)
{
  switch(msg->what)
  {
    case MsgOpenPanel:
      LaunchFilePanel();
      break;

    case MsgOpenFile:
    {
      entry_ref File;

      CloseFilePanel();

      if(msg->HasRef("refs"))
        if(msg->FindRef("refs", &File) >= B_NO_ERROR)
          OpenFile(File);
      break;
    }
    default:
      BApplication::MessageReceived(msg);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::LaunchFilePanel()                                                                        //
//                                                                                                                //
// opens the FilePanel.                                                                                           //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::LaunchFilePanel()
{
  try
  {
    if(OpenPanel)
    {
      if(!OpenPanel->IsShowing())
        OpenPanel->Show();
      return;
    }

    if(!(OpenPanel = new BFilePanel(B_OPEN_PANEL, NULL, &PanelDirectory, B_FILE_NODE, false,
                                    new BMessage(MsgOpenFile))))
      return;
    OpenPanel->Show();
  }
  catch(...)
  {
    D(bug("exception occurred at %s:%d!\n", __FILE__, __LINE__));
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::CloseFilePanel()                                                                         //
//                                                                                                                //
// stores current settings of the FilePanel.                                                                      //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::CloseFilePanel()
{
  if(OpenPanel)
  {
    OpenPanel->GetPanelDirectory(&PanelDirectory);
    OpenPanel->Hide();
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::OpenFile(entry_ref File)                                                                 //
//                                                                                                                //
// Loads a file and displays it.                                                                                  //
//                                                                                                                //
// record_ref File                      file to load                                                              //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::OpenFile(entry_ref File)
{
  BFile     *f;
  app_info  ai;
  BNodeInfo *ni;
  DVI       *NewDocument = NULL;
  char      Buffer[B_MIME_TYPE_LENGTH];
  char      *str;

  try
  {
    D(bug("ViewApplication::OpenFile()\n"));

    if(!(f = new BFile(&File, O_RDONLY)))
      return;
    if(f->InitCheck() != B_OK)
    {
      delete f;
      return;
    }

    // set icon if not already done
/*    if(ni = new BNodeInfo(&f))
    {
      if(ni->GetType(Buffer) == B_OK)
      {
        if(strcmp(Buffer, "application/octet-stream") == 0)
          ni->SetType("application/x-vnd.Be-BDVI");
      }
    }*/

    SetSleepCursor();

    if(!(NewDocument = new DVI(f, &Settings)))
    {
      DisplayError("Not enough memory!");
      return;
    }

    if(!NewDocument->Ok())
    {
      delete NewDocument;
      DisplayError("Can't load file!");
      return;
    }

    NewDocument->DisplayError = DisplayError;

    vw->SetDocument(NewDocument);

    SetNormalCursor();
  }
  catch(...)
  {
    D(bug("exception occurred at %s:%d!\n", __FILE__, __LINE__));
    if(NewDocument)
      delete NewDocument;
    SetNormalCursor();
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::DisplayError(char *msg)                                                                  //
//                                                                                                                //
// Displays an Error Requester.                                                                                   //
//                                                                                                                //
// char *msg                            message to be displayed                                                   //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::DisplayError(const char *msg)
{
  BAlert *al;

  try
  {
    if(!(al = new BAlert("Error", msg, "Ok", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT)))
      return;

    al->Go();
  }
  catch(...)
  {
    D(bug("exception occurred at %s:%d!\n", __FILE__, __LINE__));
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::SetResolution(u_int PixelsPerInch, u_int ShrinkFactor)                                   //
//                                                                                                                //
// Sets the resolution of the display.                                                                            //
//                                                                                                                //
// u_int PixelsPerInch                  dpi value (0: don't change old value)                                     //
// u_int ShrinkFactor                   factor to shrink document (0: don't change old value)                     //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::SetResolution(u_int PixelsPerInch, u_int ShrinkFactor)
{
  Settings.DspInfo.SetResolution(PixelsPerInch, ShrinkFactor);

  if(PixelsPerInch != 0)                                                       // mode may have changed
  {
    acquire_sem(kpse_sem);
    kpse_format_info[kpse_any_glyph_format].path = NULL;                       // clear paths, otherwise kpathseach
    kpse_format_info[kpse_gf_format].path        = NULL;                       // wouldn't update them
    kpse_format_info[kpse_pk_format].path        = NULL;
    kpse_format_info[kpse_vf_format].path        = NULL;

    kpse_init_prog("BEDVI", Settings.DspInfo.PixelsPerInch, Settings.DspInfo.Mode, "cmr10");
    kpse_set_program_enabled(kpse_pk_format,        1, kpse_src_compile);
    kpse_set_program_enabled(kpse_any_glyph_format, 1, kpse_src_compile);
    release_sem(kpse_sem);
  }

  if(vw)
    vw->SetDspInfo(&Settings.DspInfo);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// int main()                                                                                                     //
//                                                                                                                //
// main function.                                                                                                 //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int main()
{
  ViewApplication *ViewApp = NULL;

  try
  {
    D(bug("BeDVI " VERSION " (" __DATE__ ")\n"));

    if(!InitKpseSem())
      exit(1);

    if(!(ViewApp = new ViewApplication()))
    {
      FreeKpseSem();
      exit(1);
    }

    if(!(ViewApp->Ok()))
    {
      delete ViewApp;
      FreeKpseSem();
      exit(1);
    }

    ViewApp->Run();

    delete ViewApp;
    ViewApp = NULL;

    FreeKpseSem();

    exit(0);
  }
  catch(...)
  {
    D(bug("exception occurred at %s:%d!\n", __FILE__, __LINE__));
    if(ViewApp)
      delete ViewApp;
    FreeKpseSem();
    exit(1);
  }
}
