////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// BeDVI                                                                                                          //
// by Achim Blumensath                                                                                            //
// blume@corona.oche.de                                                                                           //
//                                                                                                                //
// This program is free software! It may be distributed according to the GNU Public License (see COPYING).        //
//                                                                                                                //
// $Id: DVI-Display.cc,v 1.4 1997/04/27 17:41:29 achim Exp achim $
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <InterfaceKit.h>
#include "debug.h"
#include "BeDVI.h"
#include "DVI-Window.h"
#include "DVI-View.h"

//static const int  AppMenuLen              = 3;
static const int  NumMenus                = 5;
static const int  NumSubMenus[NumMenus]   = {3, 2, 4, 3, 6};
static const bool RadioModeMenu[NumMenus] = {false, false, false, true, true};

static const struct
{
  char   *Name;
  char   Shortcut;
  bool   Marked;
  u_long Message;
}
/*AppMenuDef[] =
{
  {"About BeDVI...", '?', false, B_ABOUT_REQUESTED},
  {"Open...",        'O', false, MsgOpenPanel     },
//  {"Save Settings",   0,  false, MsgSaveSettings  },
  {"Quit",           'Q', false, B_QUIT_REQUESTED }
},*/
MenuDefs[] =
{
  {"File",            0,  false, NULL             },
  {"About...",       '?', false, B_ABOUT_REQUESTED},
  {"Open...",        'O', false, MsgOpenPanel     },
//  {"Save Settings",   0,  false, MsgSaveSettings  },
  {"Quit",           'Q', false, B_QUIT_REQUESTED },
  {"Display",         0,  false, NULL             },
  {"Anti Aliasing",  'A', true,  MsgAntiAliasing  },
  {"Border Line",    'B', false, MsgBorderLine    },
//  {"Measure Window", 'M', false, MsgMeasureWin    },
  {"Move",            0,  false, NULL             },
//  {"Search",         'S', false, MsgSearch        },
  {"First Page",     '<', false, MsgFirst         },
  {"Last Page",      '>', false, MsgLast          },
  {"Next Page",      '+', false, MsgNext          },
  {"Prev Page",      '-', false, MsgPrev          },
  {"Resolution",      0,  false, NULL             },
  {"600 dpi",         0,  false, 600              },
  {"300 dpi",         0,  true,  300              },
  {"100 dpi",         0,  false, 100              },
  {"Magnification",   0,  false, NULL             },
  {"Shrink to 1/1",  '1', false, 1                },
  {"Shrink to 1/2",  '2', false, 2                },
  {"Shrink to 1/3",  '3', true,  3                },
  {"Shrink to 1/4",  '4', false, 4                },
  {"Shrink to 1/5",  '5', false, 5                },
  {"Shrink to 1/6",  '6', false, 6                }
};

static u_short SleepCursorImage[] =
{
  0x1001,  // 16x16 points, depth 1
  0x0907,  // hot spot at (9, 7)
  0x03c0,  // image data
  0x07c0,
  0x0280,
  0x07e0,
  0x1838,
  0x201c,
  0x4026,
  0x4046,
  0x8083,
  0x8103,
  0x8003,
  0x4006,
  0x4006,
  0x200c,
  0x1838,
  0x07e0,
  0x07c0,  // mask data
  0x07c0,
  0x0380,
  0x07e0,
  0x1ff8,
  0x3ffc,
  0x7ffe,
  0x7ffe,
  0xffff,
  0xffff,
  0xffff,
  0x7ffe,
  0x7ffe,
  0x3ffc,
  0x1ff8,
  0x07e0,
};

char SleepPointerNestCnt = 0;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// bool InitAppMenu()                                                                                             //
//                                                                                                                //
// Creates the MainMenu.                                                                                          //
//                                                                                                                //
// Result:                              `true' if successful, otherwise `false'                                   //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*bool InitAppMenu()
{
  BMessage   *msg;
  BPopUpMenu *Menu;
  BMenuItem  *Item;
  u_int      i;

  D(bug("InitAppMenu()\n"));

  if(!(Menu = new BPopUpMenu("", false, false)))
    return false;

  for(i = 0; i < AppMenuLen; i++)
  {
    if(!(msg = new BMessage(AppMenuDef[i].Message)))
      return false;

    if(!(Item = new BMenuItem(AppMenuDef[i].Name, msg, AppMenuDef[i].Shortcut)))
    {
      delete msg;
      return NULL;
    }
    Item->SetTarget(be_app);
    Item->SetMarked(AppMenuDef[i].Marked);
    Menu->AddItem(Item);
  }

  be_app->SetMainMenu(Menu);

  return true;  
}
*/
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// static BMenuBar *InitMenu()                                                                                    //
//                                                                                                                //
// Creates the menu of a window.                                                                                  //
//                                                                                                                //
// Result:                              pointer to the menu or `NULL' if an error occures                         //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

static BMenuBar *InitMenu()
{
  BMenuBar  *MenuBar = NULL;
  BMenu     *Menu    = NULL;
  BMenuItem *Item    = NULL;
  u_int     no = 0;
  u_int     i, j;

  try
  {
    D(bug("InitMenu()\n"));

    if(!(MenuBar = new BMenuBar(BRect(0, 0, 0, 0), "Menu")))
      return NULL;

    for(i = 0; i < NumMenus; i++)
    {
      if(!(Menu = new BMenu(MenuDefs[no++].Name)))
        return NULL;
      MenuBar->AddItem(Menu);

      for(j = 0; j < NumSubMenus[i]; j++)
      {
        if(!(Item = new BMenuItem(MenuDefs[no].Name, new BMessage(MenuDefs[no].Message), MenuDefs[no].Shortcut)))
          return NULL;

        Item->SetMarked(MenuDefs[no].Marked);
        Menu->AddItem(Item);
        no++;
      }
      if(RadioModeMenu[i])
        Menu->SetRadioMode(true);
    }
  }
  catch(...)
  {
    D(bug("exception occurred at %s:%d!\n", __FILE__, __LINE__));
    if(MenuBar)
      delete MenuBar;
    MenuBar = NULL;
  }
  return MenuBar;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// ViewWindow *InitWindow()                                                                                       //
//                                                                                                                //
// Creates a window.                                                                                              //
//                                                                                                                //
// Result:                              pointer to the window or `NULL' if an error occures                       //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

ViewWindow *InitWindow()
{
  ViewWindow   *win;
  DVIView      *vw;
  BMenuBar     *Menu;
  BTextControl *PageNo;
  BScrollView  *sv;
//  BBox         *bx;
//  BScrollBar   *sb;
  BRect        MenuR;
  BRect        WindowR;
  BRect        r;

  try
  {
    D(bug("InitWindow()\n"));

    // create window

    if(!(win = new ViewWindow(((ViewApplication *)be_app)->WindowBounds)))
      return NULL;

    win->SetSizeLimits(120.0, 1e7, 120.0, 1e7);

    // create menu

    if(!(Menu = InitMenu()))
    {
      win->Quit();
      return NULL;
    }

    win->AddChild(Menu);
    win->SetKeyMenuBar(Menu);

    // get bounds of window and menu

    WindowR = win->Bounds();
    MenuR   = Menu->Frame();

    // create DVI-View

    r.Set(B_ORIGIN.x, MenuR.bottom + 1.0, WindowR.right - 14.0, WindowR.bottom - 14.0);

    if(!(vw = new DVIView(r, "DVI-View")))
    {
      win->Quit();
      return NULL;
    }
    if(!vw->Ok())
    {
      delete vw;
      win->Quit();
      return NULL;
    }

    if(!(sv = new BScrollView("Document View", vw, B_FOLLOW_ALL_SIDES, 0, true, true)))
    {
      delete vw;
      win->Quit();
      return NULL;
    }

    sv->ScrollBar(B_HORIZONTAL)->MoveBy  ( 50.0, 0.0);
    sv->ScrollBar(B_HORIZONTAL)->ResizeBy(-50.0, 0.0);

    win->AddChild(sv);
    vw->MakeFocus(true);

    // create page counter

    r = sv->ScrollBar(B_HORIZONTAL)->Frame();

    r.SetLeftTop    (sv->ConvertToParent(r.LeftTop()));
    r.SetRightBottom(sv->ConvertToParent(r.RightBottom()));

    r.right  =  r.left - 1.0;
    r.left   =  0.0;
    r.bottom -= 1.0;

    if(!(PageNo = new BTextControl(r, "PageNumber", NULL, "1", new BMessage(MsgPage), B_FOLLOW_LEFT | B_FOLLOW_BOTTOM)))
    {
      win->Quit();
      return NULL;
    }
    PageNo->SetTarget(win);
    PageNo->SetAlignment(B_ALIGN_LEFT, B_ALIGN_RIGHT);
    PageNo->SetLowColor(219, 219, 219);
    PageNo->SetViewColor(219, 219, 219);

    BFont font(be_fixed_font);
    font.SetSize(9);
    PageNo->TextView()->SetFontAndColor(&font, B_FONT_ALL);

    win->AddChild(PageNo);
    PageNo->ResizeTo(r.Width(), r.Height());
    PageNo->ResizeTo(r.Width(), r.Height());
    PageNo->SetText("1");
  }
  catch(...)
  {
    D(bug("exception occurred at %s:%d!\n", __FILE__, __LINE__));
    if(win)
      win->Quit();
    win = NULL;
  }

  return win;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void SetSleepCursor()                                                                                          //
//                                                                                                                //
// Sets the cursor image to a sleeping-cloud.                                                                     //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SetSleepCursor()
{
  if(SleepPointerNestCnt++ < 1)
    be_app->SetCursor(SleepCursorImage);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void SetNormalCursor()                                                                                         //
//                                                                                                                //
// Sets the cursor image to the normal Be-hand.                                                                   //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SetNormalCursor()
{
  if(--SleepPointerNestCnt < 1)
    be_app->SetCursor(B_HAND_CURSOR);
}
