////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// BeDVI                                                                                                          //
// by Achim Blumensath                                                                                            //
// blume@corona.oche.de                                                                                           //
//                                                                                                                //
// This program is free software! It may be distributed according to the GNU Public License (see COPYING).        //
//                                                                                                                //
// $Id: DVI.h,v 1.4 1997/04/27 17:41:29 achim Exp achim $
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef DVI_H
#define DVI_H

#include <StorageKit.h>

#ifndef DEFINES_H
#include "defines.h"
#endif
#ifndef LIST_H
#include "list.h"
#endif
#ifndef FONTLIST_H
#include "FontList.h"
#endif

class  BPositionIO;
class  BView;
struct DrawInfo;
class  DVIView;
class  Font;

typedef long (*SetCharProc)(DrawInfo *, u_char);

#ifdef __MWERKS__
  static const u_int        NumModes            = 3;
  static const char * const ModeNames[NumModes] = {"nextscrn", "cx", "ljfour"};
  static const u_int        ModeDPI[NumModes]   = {100, 300, 600};
#endif

class DisplayInfo
{
  private:

#ifndef __MWERKS__
    // modes sorted by increasing resolution

    static const u_int        NumModes            = 3;
    static const char * const ModeNames[NumModes] = {"nextscrn", "cx", "ljfour"};
    static const u_int        ModeDPI[NumModes]   = {100, 300, 600};
#endif

  public:
    const char *Mode;
    u_int      PixelsPerInch;
    u_int      ShrinkFactor;

    DisplayInfo():
      Mode(ModeNames[NumModes / 2]),
      PixelsPerInch(ModeDPI[NumModes / 2]),
      ShrinkFactor(ModeDPI[NumModes / 2] / 100)
    {}

    void SetResolution(u_int ppi, u_int sf);

    long ToPixel(long x)
    {
      return (x + (ShrinkFactor << 16) - 1) / (ShrinkFactor << 16);
    }

    long PixelConv(long x)
    {
      return (x / ShrinkFactor) >> 16;
    }
};

class DVISettings
{
  public:
    DisplayInfo DspInfo;
    bool        AntiAliasing;
    bool        BorderLine;
};

struct FrameData
{
  long Horiz;
  long Vert;
  long w, x, y, z;
  int  PixelV;
};

class Frame: public Node
{
  public:
    FrameData Data;
};

class DrawInfo
{
  public:
    DVI         *Document;
    BView       *vw;          // view to draw into
    FrameData   Data;         // current position
    DisplayInfo *DspInfo;
    double      DimConvert;
    Font        *CurFont;     // current font
    FontTable   *VirtTable;
    Font        *Virtual;
    u_char      MaxChar;
    SetCharProc SetChar;
    List        FrameList;
    List        FreeFrames;
    BPositionIO *File;        // DVI file
    u_char      *BufferPos;   // for buffered I/O
    u_char      *BufferEnd;

    u_long ReadInt (ssize_t Size);
    long   ReadSInt(ssize_t Size);
    void   ReadString(char *Buffer, size_t len);
    void   Skip(off_t off);
};

#ifdef __MWERKS__
  static const u_char SetChar0  =   0;
  static const u_char Set1      = 128;
  static const u_char SetRule   = 132;
  static const u_char Put1      = 133;
  static const u_char PutRule   = 137;
  static const u_char NOP       = 138;
  static const u_char BeginOP   = 139;
  static const u_char EndOP     = 140;
  static const u_char Push      = 141;
  static const u_char Pop       = 142;
  static const u_char Right1    = 143;
  static const u_char Right2    = 144;
  static const u_char Right3    = 145;
  static const u_char Right4    = 146;
  static const u_char W0        = 147;
  static const u_char W1        = 148;
  static const u_char W2        = 149;
  static const u_char W3        = 150;
  static const u_char W4        = 151;
  static const u_char X0        = 152;
  static const u_char X1        = 153;
  static const u_char X2        = 154;
  static const u_char X3        = 155;
  static const u_char X4        = 156;
  static const u_char Down1     = 157;
  static const u_char Down2     = 158;
  static const u_char Down3     = 159;
  static const u_char Down4     = 160;
  static const u_char Y0        = 161;
  static const u_char Y1        = 162;
  static const u_char Y2        = 163;
  static const u_char Y3        = 164;
  static const u_char Y4        = 165;
  static const u_char Z0        = 166;
  static const u_char Z1        = 167;
  static const u_char Z2        = 168;
  static const u_char Z3        = 169;
  static const u_char Z4        = 170;
  static const u_char FontNum0  = 171;
  static const u_char Font1     = 235;
  static const u_char Font2     = 236;
  static const u_char Font3     = 237;
  static const u_char Font4     = 238;
  static const u_char XXX1      = 239;
  static const u_char XXX2      = 240;
  static const u_char XXX3      = 241;
  static const u_char XXX4      = 242;
  static const u_char FontDef1  = 243;
  static const u_char FontDef2  = 244;
  static const u_char FontDef3  = 245;
  static const u_char FontDef4  = 246;
  static const u_char Preamble  = 247;
  static const u_char Postamble = 248;
  static const u_char PostPost  = 249;
  static const u_char StartRefl = 250;
  static const u_char EndRefl   = 251;
  static const u_char Trailer   = 223;
#endif

class DVI
{
  private:

#ifndef __MWERKS__
    static const u_char SetChar0  =   0;
    static const u_char Set1      = 128;
    static const u_char SetRule   = 132;
    static const u_char Put1      = 133;
    static const u_char PutRule   = 137;
    static const u_char NOP       = 138;
    static const u_char BeginOP   = 139;
    static const u_char EndOP     = 140;
    static const u_char Push      = 141;
    static const u_char Pop       = 142;
    static const u_char Right1    = 143;
    static const u_char Right2    = 144;
    static const u_char Right3    = 145;
    static const u_char Right4    = 146;
    static const u_char W0        = 147;
    static const u_char W1        = 148;
    static const u_char W2        = 149;
    static const u_char W3        = 150;
    static const u_char W4        = 151;
    static const u_char X0        = 152;
    static const u_char X1        = 153;
    static const u_char X2        = 154;
    static const u_char X3        = 155;
    static const u_char X4        = 156;
    static const u_char Down1     = 157;
    static const u_char Down2     = 158;
    static const u_char Down3     = 159;
    static const u_char Down4     = 160;
    static const u_char Y0        = 161;
    static const u_char Y1        = 162;
    static const u_char Y2        = 163;
    static const u_char Y3        = 164;
    static const u_char Y4        = 165;
    static const u_char Z0        = 166;
    static const u_char Z1        = 167;
    static const u_char Z2        = 168;
    static const u_char Z3        = 169;
    static const u_char Z4        = 170;
    static const u_char FontNum0  = 171;
    static const u_char Font1     = 235;
    static const u_char Font2     = 236;
    static const u_char Font3     = 237;
    static const u_char Font4     = 238;
    static const u_char XXX1      = 239;
    static const u_char XXX2      = 240;
    static const u_char XXX3      = 241;
    static const u_char XXX4      = 242;
    static const u_char FontDef1  = 243;
    static const u_char FontDef2  = 244;
    static const u_char FontDef3  = 245;
    static const u_char FontDef4  = 246;
    static const u_char Preamble  = 247;
    static const u_char Postamble = 248;
    static const u_char PostPost  = 249;
    static const u_char StartRefl = 250;
    static const u_char EndRefl   = 251;
    static const u_char Trailer   = 223;
#endif

    BPositionIO *DVIFile;
    char        *Name;
    double      TPicConvert;
    int         OffsetX;
    int         OffsetY;
    u_int       NumPages;
    u_int       UnshrunkPageWidth;
    u_int       UnshrunkPageHeight;
    u_long      *PageOffset;
    FontTable   Fonts;

  public:
    void        (*DisplayError)(const char *str);
    DVISettings Settings;
    long        Magnification;
    double      DimConvert;
    u_int       PageWidth;
    u_int       PageHeight;

  private:
    void ChangeFont(DrawInfo *di, u_long n);
    void Special(DrawInfo *di, long len);

    static long SetNoChar(DrawInfo *di, u_char c);
    static long SetChar(DrawInfo *di, u_char c);
    static long SetVFChar(DrawInfo *di, u_char c);
    static void DrawRule(DrawInfo *di, long x, long y, long w, long h);

  public:
    DVI(BPositionIO *File, DVISettings *prefs);
    ~DVI();

    bool Reload();
    void SetDspInfo(DisplayInfo *dsp);
    void Draw(BView *vw, u_int PageNo);
    void DrawPart(DrawInfo *di);
    int  MagStepValue(float &mag);

    bool Ok()
    {
      return Fonts.Ok() && DVIFile != NULL && PageOffset != NULL;
    }

    friend class DVIView;
    friend class FontTable;
    friend class Font;
};

#endif
