////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// BeDVI                                                                                                          //
// by Achim Blumensath                                                                                            //
// blume@corona.oche.de                                                                                           //
//                                                                                                                //
// This program is free software! It may be distributed according to the GNU Public License (see COPYING).        //
//                                                                                                                //
// $Id: GF.cc,v 1.2 1996/12/26 15:34:26 achim Exp $
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <string.h>
#include <stdio.h>
#include "debug.h"
#include "BeDVI.h"
#include "TeXFont.h"

static const u_char GF_Paint0    = 0;
static const u_char GF_Paint1    = 64;
static const u_char GF_Paint2    = 65;
static const u_char GF_Paint3    = 66;
static const u_char GF_BOC       = 67;
static const u_char GF_BOC1      = 68;
static const u_char GF_EOC       = 69;
static const u_char GF_Skip0     = 70;
static const u_char GF_Skip1     = 71;
static const u_char GF_Skip2     = 72;
static const u_char GF_Skip3     = 73;
static const u_char GF_NewRow0   = 74;
static const u_char GF_NewRowMax = 238;
static const u_char GF_XXX1      = 239;
static const u_char GF_XXX2      = 240;
static const u_char GF_XXX3      = 241;
static const u_char GF_XXX4      = 242;
static const u_char GF_YYY       = 243;
static const u_char GF_NOP       = 244;
static const u_char GF_CharLoc   = 245;
static const u_char GF_CharLoc0  = 246;
static const u_char GF_Post      = 248;
static const u_char GF_PostPost  = 249;
static const u_char GF_Trailer   = 223;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// static void ReadChar(Font *f, u_char c)                                                                        //
//                                                                                                                //
// Reads a character from a font-file.                                                                            //
//                                                                                                                //
// Font   *f                            font                                                                      //
// u_char c                             character                                                                 //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

static void ReadChar(Font *f, u_char c)
{
  Glyph  *g;
  u_char Cmd;
  int    MinM, MaxM;
  int    MinN, MaxN;
  u_char *cp;
  u_char *BaseP;
  u_char *MaxP;
  int    BytesWide;
  bool   PaintSwitch;
  bool   NewRow;
  int    Count;
  int    WordWeight;
  u_long Width, Height;

  try
  {
    g = &f->Glyphs[c];

    while(true)
    {
      switch(Cmd = ReadInt(&f->File, 1))
      {
        case GF_XXX1:
        case GF_XXX2:
        case GF_XXX3:
        case GF_XXX4:
          f->File.Seek(ReadInt(&f->File, Cmd - GF_XXX1 + 1), SEEK_CUR);
          continue;

        case GF_YYY:
          f->File.Seek(4, SEEK_CUR);
          continue;

        case GF_BOC:
          f->File.Seek(8, SEEK_CUR);

          MinM = ReadSInt(&f->File, 4);
          MaxM = ReadSInt(&f->File, 4);
          MinN = ReadSInt(&f->File, 4);
          MaxN = ReadSInt(&f->File, 4);

          g->Ux = -MinM;
          g->Uy =  MaxN;

          Width  = MaxM - MinM + 1;
          Height = MaxN - MinN + 1;
          break;

        case GF_BOC1:
          f->File.Seek(1, SEEK_CUR);

          Width = ReadInt(&f->File, 1);
          g->Ux = Width - ReadInt(&f->File, 1);

          Width++;

          Height = ReadInt(&f->File, 1);
          g->Uy  = ReadInt(&f->File, 1);
          break;

        default:
          return;
      }
      break;
    }
    PaintSwitch = 0;

    if(!(g->UBitMap = new BBitmap(BRect(0.0, 0.0, (float)Width - 1.0, (float)Height - 1.0), B_MONOCHROME_1_BIT)))
      return;

    BaseP      = (u_char *)g->UBitMap->Bits();
    cp         = BaseP;
    BytesWide  = g->UBitMap->BytesPerRow();
    MaxP       = BaseP + g->UBitMap->BitsLength();
    NewRow     = false;
    WordWeight = 8;

    bzero(g->UBitMap->Bits(), g->UBitMap->BitsLength());

    while(true)
    {
      Count = -1;

      Cmd = ReadInt(&f->File, 1);

      if(Cmd < 64)
        Count = Cmd;
      else if(Cmd > GF_NewRow0 && Cmd <= GF_NewRowMax)
      {
        Count       = Cmd - GF_NewRow0;
        PaintSwitch = 0;
        NewRow      = true;
      }
      else
        switch(Cmd)
        {
          case GF_Paint1:
          case GF_Paint2:
          case GF_Paint3:
            Count = ReadInt(&f->File, Cmd - GF_Paint1 + 1);
            break;

          case GF_EOC:
            return;

          case GF_Skip1:
          case GF_Skip2:
          case GF_Skip3:
            BaseP += ReadInt(&f->File, Cmd - GF_Skip0) * BytesWide;

          case GF_Skip0:
            NewRow      = true;
            PaintSwitch = 0;
            break;

          case GF_XXX1:
          case GF_XXX2:
          case GF_XXX3:
          case GF_XXX4:
            f->File.Seek(ReadInt(&f->File, Cmd - GF_XXX1 + 1), SEEK_CUR);
            break;

          case GF_YYY:
            f->File.Seek(4, SEEK_CUR);
            break;

          case GF_NOP:
            break;

          default:
            return;
        }
      if(NewRow)
      {
        BaseP     += BytesWide;
        cp         = BaseP;
        WordWeight = 8;
        NewRow     = false;
      }
      if(Count >= 0)
      {
        while(Count)
          if(Count <= WordWeight)
          {
            WordWeight -= Count;

            if(PaintSwitch)
              *cp |= BitMasks[Count] << WordWeight;

            break;
          }
          else
          {
            if(PaintSwitch)
              *cp |= BitMasks[WordWeight];

            cp++;
            Count     -= WordWeight;
            WordWeight = 8;
          }
        PaintSwitch = 1 - PaintSwitch;
      }
    }
  }
  catch(...)
  {
    D(bug("exception occurred at %s:%d!\n", __FILE__, __LINE__));
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// bool ReadGFIndex(DVI *doc, Font *f)                                                                            //
//                                                                                                                //
// Reads general information from a font-file.                                                                    //
//                                                                                                                //
// DVI  *doc                            document the font appears in                                              //
// Font *f                              font                                                                      //
//                                                                                                                //
// Result:                              `true' if successful, otherwise `false'                                   //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool ReadGFIndex(DVI *doc, Font *f)
{
  int    hppp, vppp;
  u_char c, Cmd;
  Glyph  *g;
  long   CheckSum;

  f->File.Seek(-4, SEEK_END);

  while(ReadInt(&f->File, 4) != ((u_long)GF_Trailer << 24 | GF_Trailer << 16 | GF_Trailer << 8 | GF_Trailer))
    f->File.Seek(-5, SEEK_CUR);

  f->File.Seek(-5, SEEK_CUR);

  for(c = ReadInt(&f->File, 1); c == GF_Trailer; f->File.Seek(-2, SEEK_CUR))
    ;

  if(c != GF_ID)
    return false;

  f->File.Seek(-6, SEEK_CUR);

  if(ReadInt(&f->File, 1) != GF_PostPost)
    return false;

  f->File.Seek(ReadSInt(&f->File, 4), SEEK_SET);

  if(ReadInt(&f->File, 1) != GF_Post)
    return false;

  f->File.Seek(8, SEEK_CUR);

  CheckSum = ReadInt(&f->File, 4);

  hppp = ReadSInt(&f->File, 4);
  vppp = ReadSInt(&f->File, 4);

  f->File.Seek(16, SEEK_CUR);

  if(!(f->Glyphs = new Glyph[256]))
    return false;

  while((Cmd = ReadInt(&f->File, 1)) != GF_PostPost)
  {
    long Addr;

    c = ReadInt(&f->File, 1);

    g = &f->Glyphs[c];

    switch(Cmd)
    {
      case GF_CharLoc:
        f->File.Seek(8, SEEK_CUR);
        break;

      case GF_CharLoc0:
        f->File.Seek(4, SEEK_CUR);
        break;

      default:
        return false;
    }
    g->Advance = f->DimConvert * ReadSInt(&f->File, 4);

    if((Addr = ReadInt(&f->File, 4)) != -1)
      g->Addr = Addr;
  }

  f->ReadChar = ::ReadChar;

  return true;
}
