////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// BeDVI                                                                                                          //
// by Achim Blumensath                                                                                            //
// blume@corona.oche.de                                                                                           //
//                                                                                                                //
// This program is free software! It may be distributed according to the GNU Public License (see COPYING).        //
//                                                                                                                //
// $Id$
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <StorageKit.h>
#include "Support.h"

sem_id kpse_sem = B_ERROR;  // kpathsearch library isn't thread safe

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// u_long ReadInt(BPositionIO *f, ssize_t Size)                                                                   //
//                                                                                                                //
// Reads an integer of `Size' bytes from `f'.                                                                     //
//                                                                                                                //
// BPositionIO *f                       file                                                                      //
// ssize_t     Size                     size of the integer in bytes                                              //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

u_long ReadInt(BPositionIO *f, ssize_t Size)
{
  u_long x = 0;

  f->Read((u_char *)&x + sizeof(u_long) - Size, Size);

  return x;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// long ReadSInt(BPositionIO *f, ssize_t Size)                                                                    //
//                                                                                                                //
// Reads a signed integer of `Size' bytes from `f'.                                                               //
//                                                                                                                //
// BPositionIO *f                       file                                                                      //
// ssize_t     Size                     size of the integer in bytes                                              //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

long ReadSInt(BPositionIO *f, ssize_t Size)
{
  long x = 0;

  f->Read((u_char *)&x + sizeof(long) - 1, 1);

  if(x & 0x80)
    x |= 0xffffff00;

  x <<= (Size - 1) * 8;

  if(Size > 1)
    f->Read((u_char *)&x + sizeof(long) - Size + 1, Size - 1);

  return x;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// bool InitKpseSem()                                                                                             //
//                                                                                                                //
// initializes `kpse_sem'.                                                                                        //
//                                                                                                                //
// Result:                              `true' if successful otherwise `false'                                    //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool InitKpseSem()
{
  return (kpse_sem = create_sem(1, "kpse semaphore")) >= B_OK;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void FreeKpseSem()                                                                                             //
//                                                                                                                //
// frees `kpse_sem'.                                                                                              //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FreeKpseSem()
{
  if(kpse_sem >= B_OK)
  {
    acquire_sem(kpse_sem);
    delete_sem(kpse_sem);
  }
}
