////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// BeDVI                                                                                                          //
// by Achim Blumensath                                                                                            //
// blume@corona.oche.de                                                                                           //
//                                                                                                                //
// This program is free software! It may be distributed according to the GNU Public License (see COPYING).        //
//                                                                                                                //
// $Id: VF.cc,v 1.2 1996/12/26 15:34:26 achim Exp $
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <string.h>
#include "BeDVI.h"
#include "FontList.h"
#include "TeXFont.h"
#include "DVI.h"
#include "DVI-View.h"

static const u_char LongChar = 242;

static const u_int VF_Param_1 = 20;
static const u_int VF_Param_2 = 256;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// bool ReadVFIndex(DVI *doc, Font *f)                                                                            //
//                                                                                                                //
// Reads general information from a font-file.                                                                    //
//                                                                                                                //
// DVI  *doc                            document the font appears in                                              //
// Font *f                              font                                                                      //
//                                                                                                                //
// Result:                              `true' if successful, otherwise `false'                                   //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool ReadVFIndex(DVI *doc, Font *f)
{
  u_char Cmd;
  u_char *Avail;
  u_char *AvailEnd;
  long   CheckSum;

  f->ReadChar = NULL;
  f->Virtual  = true;

  f->File.Seek(ReadInt(&f->File, 1), SEEK_CUR);

  CheckSum = ReadInt(&f->File, 4);

  f->File.Seek(4, SEEK_CUR);

  f->FirstFont = NULL;

  for(Cmd = ReadInt(&f->File, 1); Cmd >= FontDef1 && Cmd <= FontDef4; Cmd = ReadInt(&f->File, 1))
  {
    Font        *NewFont;
//    BFile       *FileCopy;
//    DFileStream *file;
//
//    if(!(FileCopy = new BFile(f->File)))                // f->File must be copied, as DFileStream deletes
//      return false;                                     //  the file passed to it
//
//    if(!(file = new DFileStream(FileCopy, 0)))
//    {
//      delete FileCopy;
//      return false;
//    }
    if(!(NewFont = f->VFTable.LoadFont(doc, &f->File, Cmd)))
    {
//      delete file;
      return false;
    }

    if(f->FirstFont == NULL)
      f->FirstFont = NewFont;
  }
  if(!(f->Macros = new Macro[256]))
    return false;

  bzero(f->Macros, 256 * sizeof(Macro));

  Avail    = NULL;
  AvailEnd = NULL;

  for(; Cmd <= LongChar; Cmd = ReadInt(&f->File, 1))
  {
    Macro  *m;
    int    len;
    u_long cc;
    long   Width;

    if(Cmd == LongChar)
    {
      len   = ReadInt(&f->File, 4);
      cc    = ReadInt(&f->File, 4);
      Width = ReadInt(&f->File, 4);

      if(cc >= 256)
      {
        f->File.Seek(len, SEEK_CUR);
        continue;
      }
    }
    else
    {
      len   = Cmd;
      cc    = ReadInt(&f->File, 1);
      Width = ReadInt(&f->File, 3);
    }
    m = &f->Macros[cc];

    m->Advance = Width * f->DimConvert;

    if(len > 0)
    {
      if(len <= AvailEnd - Avail)
      {
        m->Position = Avail;
        Avail      += len;
      }
      else
      {
        m->FreeMe = true;

        if(len <= VF_Param_1)
        {
          if(!(m->Position = new u_char[VF_Param_2]))
            return false;
          Avail    = m->Position;
          AvailEnd = Avail + VF_Param_2;
          Avail   += len;
        }
        else
          if(!(m->Position = new u_char[len]))
            return false;
      }
      f->File.Read(m->Position, len);

      m->End = m->Position + len;
    }
  }

  if(Cmd != Postamble)
    return false;

  return true;
}
